from typing import Callable, Any

class Adapter:
    def __call__(self, prompt: str) -> str:
        """
        Accepts a prompt or input as a string and returns the corresponding LLM response or output as a string.
        """
        raise NotImplementedError("Adapter is an abstract base class and cannot be instantiated directly.")

class CustomAdapter(Adapter):

    # custom llm must be calllable and always return string as prompt
    # can take the input of any or stringa and return string. 
    def __init__(self, custom_llm: Callable[[Any], str]):
        self._validate_callable(custom_llm)
        self.custom_llm = custom_llm

    def __call__(self, prompt: str) -> tuple:
        # Implement custom LLM logic and processing steps
        llm_response = self.custom_llm(prompt)
        return prompt, llm_response

    @staticmethod
    # check if its callable
    def _validate_callable(obj: Callable[[str], str]):
        if not callable(obj):
            raise TypeError("The 'custom_llm' argument must be a callable object that accepts a string and returns a string.")


# without adapter we basicaly cant work
def createAdapterOutOf(fn:Callable[[Any] , str]) -> CustomAdapter : 
    try : 
        res = CustomAdapter(fn);
        return res;
    except TypeError as e  : 
        print('cant create adapter out of the provided callable llm Wraper need fn(any) -> string : wrap all your llm logic to this fn signature');
        exit(1);

def main ()  : 

    try:
        # Instantiate CustomAdapter with the custom_llm_function
        ca = CustomAdapter(custom_llm_function)
        # Call the adapter with a prompt
        res = ca("Input prompt")
        print(res)
    except TypeError as e:
        print("Can't create it:", e)

    try : 
        ca = CustomAdapter(CustomWrapper);
        res = ca({'dicti':'good'});
        print(res);
    except TypeError : 
        print('cant create it ')



def CustomWrapper(good:Any) -> str : 
    print(good)
    return 'this is custom wrapper'


def custom_llm_function(prompt: str) -> str:
    # Implement your custom logic here
    return "Custom logic applied: " + prompt


if __name__ == "__main__" : 
    main();


