# Written and Refactor by keihan haqiq github@patpaws
# Utilities for Working with jsonStuffs

# import libs
import json
# for handling static typing
from typing import Any, Dict
from pathlib import Path

def dataLoader(fpath: str) -> Dict[str, Any]:
    """
    Load data from a JSON file.

    Args:
        fpath (str): Path to the JSON file.

    Returns:
        Dict[str, Any]: Data loaded from the JSON file.
    """
    try:
        with Path(fpath).open('r', encoding='utf-8') as f:
            return json.load(f)
    except (json.JSONDecodeError, FileNotFoundError, PermissionError) as e:
        print(f"Error loading JSON file: {e}")
        return {}

def writeJsonFile(path: str, value: Any) -> None:
    """
    Write data to a JSON file.

    Args:
        path (str): Path to the JSON file.
        value (Any): Data to be written to the file.
    """
    try:
        with Path(path).open('w', encoding='utf-8') as json_file:
            json.dump(value, json_file, indent=2)
            return True;
    except (PermissionError, OSError) as e:
        print(f"Error writing to JSON file: {e}")
        return False

def main() -> None:
    """
    Test the JSON loader.
    """
    data = dataLoader('../testData/testData.json')
    if data:
        print(f"Length of data: {len(data)}")
        print(f"First item: {data[0]}")

if __name__ == "__main__":
    main()
