# Written and Refactor by keihan haqiq github@patpaws
#The MKC class is responsible for handling user input, checking the cache for exact and similar matches, and updating the cache when necessary.
# The MKC class uses the Similarity class to generate and compare signatures.
# The MKC class maintains counters for cache hits and misses, and provides a method to get the overall cache statistics
# The MKC class uses the lur class to manage Lru
#  The MKC class uses the processor class to handle preprocess and post process tasks on prompt
from minCache.lru import LRUCache
from minCache.dispatchPromptObject import PromptCached
from minCache.copy import copy
from minCache.minhash import generateSignature , checkForSimilarity;
from minCache.llmSampleCall import llmCall
from minCache.processor import Processor
from minCache.sim_generator import Similarity
from minCache.adapter import createAdapterOutOf
import time

class MINCACHE () :
    def __init__(self,cacheSize=10,threshold=0.7,llmAdapter=llmCall,withWhat='minHash',perm=200) :
        self.cache = LRUCache(cacheSize);
        self.threshold = threshold
        self.adapter = llmAdapter # Treat the adapter as a property that the user must create a vllm instance and put it here
        self.cacheHit = 0;
        self.cacheMiss = 0
        self.processor = Processor();
        self.similarEngine = Similarity(withWhat=withWhat , perm=perm) # You can add plugins by chaining okn the processor object
        # use for keep track of performance of the system


    def incrCacheHit(self) :
        self.cacheHit += 1


    def incrCacheMiss(self) :
        self.cacheMiss += 1


    def totalCacheStat(self) :
        return  {
            'cacheHitTotal': self.cacheHit,
            'cacheMissTotal': self.cacheMiss,
            'hitRate': f'{int((self.cacheHit / (self.cacheHit + self.cacheMiss)) * 100)}%',
            'missRate': f'{int((self.cacheMiss / (self.cacheHit + self.cacheMiss)) * 100)}%',
        }


    def handle(self,prompt) :
        # generate the rep for prompt
        sig = self.similarEngine.generateRep(prompt.strip().lower());
        # sign it
        promptSign = self.processor.PreProcess(prompt);
        if not promptSign :
            return self.processor.PostProcess( {
                'value':['cant handle'] ,
                'typi' : ['error']
            });

        # the promptSign : hash you got here is completly preprocessed


        # Check for the exact match (hard compare) first
        if res := self.checkAgainstHard(promptSign) :
            self.incrCacheHit();
            return self.processor.PostProcess({
                'value':[res] ,
                'typi':'exact'
            })


        # by combining the hard and semantic this is where hybrid fast approach comes to play
        # check for semantic similarity
        valids = self.checkAgainstCache(sig);
        if not valids  :
            # need to update the lru
            # do llm stuffs.
            p , r = self.adapter(prompt);
            if (r and r.strip().lower != '')  :
            # update the lru
                self.cache.put(promptSign,PromptCached(p, r , sig));
            else :

                self.incrCacheMiss()
                return self.processor.PostProcess ({

                    'value' : ['cant handle null respond from llm'],
                    'typi':['error']
                })

            self.incrCacheMiss()
            # return the new reponse to the user


            return self.processor.PostProcess( {
                'value':r ,
                'typi' : 'llm-raw'

            })
        else :
            self.incrCacheHit();
            # for now return all the valids.
            return self.processor.PostProcess( {

                'value' : valids ,
                'typi': 'cache-out'

            })
            # return the valid reponse to the user.


    def checkAgainstHard(self,targetHash) :
        cache_copy = copy(self.cache._cache)
        for cacheValue in cache_copy :
            if targetHash == cacheValue :
                return  {
                    'key': cacheValue ,
                    'value':cache_copy.get(cacheValue)
                }

        return False

    def checkAgainstCache(self,targetSig) :
        valids = [];
        cache_copy = copy(self.cache._cache)
        for cacheValue in cache_copy :
            cacheObject = cache_copy.get(cacheValue);
            scoreValue = self.similarEngine.checkAgainst(targetSig , cacheObject.getPromptRep());
            print('------------------------------------- scorevalue is ' , scoreValue);
            if (float(scoreValue) >= float(self.threshold)) :
                valids.append({'key':cacheValue , 'value':cacheObject , 'score':scoreValue});
        v = sorted(valids, key=lambda x: x['score'], reverse=True)
        if len(v) == 0 :
            return []
        print(v[0]);
        return sorted(valids, key=lambda x: x['score'], reverse=True)



def main() :
    adt = createAdapterOutOf(llmCall)
    semCache = MINCACHE(100,0.77,adt, perm=100)

    while True:
        userInput = input(" what is your root symptom > ");
        if userInput == 'exit':
            return;
        res = semCache.handle(userInput);
        print(res);
        semCache.cache.monitor();
        # for seeing the total cache stats
        print(semCache.totalCacheStat())

        print(res);
    pass



if __name__ == "__main__" :
    main();
